
# Version 1.2
#
#************************************
# Install or loading required packages
#************************************
source("required_libraries.R")

#************************************
# Loading required function
#************************************
source("all_function.R")

#************************************
# Download ILSA data with EdSurvey package
#************************************
# This program is designed to work with PISA (2015, 2018), TIMSS (2015, 2019) and PIRLS (2016) data.
# Files are placed in a folder named [ILSA]/[years].
# for Windows "C:/Users/", for Mac "/Users/"

#=* PISA *=#
myroot <- "C:/Users/mif04069/OneDrive - Vilnius University/Documents/R/S_EFECTAS/" #write your own path
ILSA <- "PISA"
year <- 2018
downloadPISA(myroot, ILSA, year)

#=* TIMSS *=#
myroot <- "C:/Users/mif04069/OneDrive - Vilnius University/Documents/R/S_EFECTAS/" #write your own path
ILSA <- "TIMSS"
year <- 2019
downloadPISA(myroot, ILSA, year)

#=* PIRLS *=#
myroot <- "C:/Users/mif04069/OneDrive - Vilnius University/Documents/R/S_EFECTAS/" #write your own path
ILSA <- "PIRLS"
year <- 2016
downloadPISA(myroot, ILSA, year)

#************************************
# Preparation of data for statistical analysis
#************************************

#=* PISA *=#
# names for plausible values PISA "math", "read", "scie", "scep", "sced", "scid", "skco", "skpe", "ssph", "ssli", "sses", "flit", "clps"
mycountry <- c("LTU")
myvariables <- c("math", "read", "scie","escs", "hisei", "st011q12ta", "st004d01t", "st034q02ta", "sc012q01ta")
my_pisa_data <- data_form(myroot, ILSA, year, mycountry, myvariables)
summary(my_pisa_data) #brief information about each column
colnames(my_pisa_data) #column names

#=* TIMSS *=#
# names for plausible values TIMSS "mmat", "ssci", "mnum", "mgeo", "mdat", "mkno", "mapp", "mrea", "slif", "sphy", "sear", "skno", "sapp", "srea"
mycountry <- c("LTU")
myvariables <- c("mmat","asbg01","asdhedup", "asdhaps","asbg04")
my_timss_data <- data_form(myroot, ILSA, year, mycountry, myvariables)
summary(my_timss_data) #brief information about each column
colnames(my_timss_data) #column names

#=* PIRLS *=#
# names for plausible values PIRLS "rrea", "rlit", "rinf", "riie", "rrsi"
mycountry <- c("LTU")
myvariables <- c("rrea","asbg01","asdhedup", "asbg04")
my_pirls_data <- data_form(myroot, ILSA, year, mycountry, myvariables)
summary(my_pirls_data) #brief information about each column
colnames(my_pirls_data) #column names


# Data preparation according to OECD manual
change <- c("FEMALE", 1, "MALE", 0)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "st004d01t", newname = "gender", change = change)
change <- c("NO RESPONSE", 9, "OTHER COUNTRY", 2, "COUNTRY OF TEST", 1)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "st019aq01t", change = change)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "st019bq01t", change = change)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "st019cq01t", change = change)
mydata1$immig <- (100*mydata1$st019aq01t)+(10*mydata1$st019bq01t)+(mydata1$st019cq01t)
change <- c(111, 0, 121, 0, 112, 0, 122, 1, 222, 1)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "immig", change = change)
mydata1$st019aq01t <- NULL
mydata1$st019bq01t <- NULL
mydata1$st019cq01t <- NULL
change <- c("GENERAL", 0, "PRE-VOCATIONAL", 1, "VOCATIONAL", 1, "MODULAR", 1)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "iscedo", newname = "vocation", change = change)
mydata1 <- na.omit(mydata1)

# Weight normalization
mydata1 <- normalization_weight(mydata1)
mydata1$mu_escs <- ave(mydata1[c("escs", "w_fsturwt0")], mydata1$cntschid, FUN=function(x) weighted.mean(x$escs, x$w_fsturwt0))[[1]]
mydata1$pct_im <- ave(mydata1[c("immig", "w_fsturwt0")], mydata1$cntschid, FUN=function(x) weighted.mean(x$immig, x$w_fsturwt0))[[1]]
mydata1$type <- mydata1$vocation


#************************************
# Descriptive statistic
#************************************

# Frequency Table and percentage for categorical variables

# 1 example
dsc_variables1 <-  c("st011q12ta", "st004d01t", "st034q02ta", "sc012q01ta")
(table1 <- frequency_table(mydata, myvariables, dsc_variables1))

# 2 example
dsc_variables1 <-  c("st011q12ta", "st034q02ta")
dsc_group <- c("st004d01t")
(table2 <- frequency_table(mydata, myvariables, dsc_variables1, mygroup = dsc_group))

# 3 example
dsc_variables1 <-  c("st011q12ta", "st034q02ta", " escs")
dsc_group <- c("st004d01t")
unique = TRUE
(table3 <- frequency_table(mydata, myvariables, dsc_variables1, mygroup = dsc_group, uniq = unique))

# 4 example
# only for one variable!!!
dsc_variables1 <-  c("st011q12ta")
missing_values = TRUE
(table4 <- frequency_table(mydata, myvariables, dsc_variables1, missing_values = missing_values))

# Min, Max, Average, Standard deviation, Median, Quartile for ordinal variables
dsc_variables2 <- c("escs", "pv1scie", "pv2scie", "pv3scie", "pv4scie", "pv5scie", "pv6scie", "pv7scie", "pv8scie",
                "pv9scie", "pv10scie", "scie")
(table5 <- descriptive(dsc_variables2))

# Percentiles
prc_variables <- "escs"
percent <- c(5, 25, 50, 75, 95)
(percentiles <- tile(prc_variables, percent))

# Benchmarks only for all plausible values
bn_variable <- "scie"
bench <- c(357.77, 420.07,482.38, 544.68, 606.99, 669.30)
(marks <- ben_marks(mydata, bn_variable, bench))


#************************************
# Correlation
#************************************

# 1 example
cor_variables <- c("escs","pv1scie", "pv2scie")
(cor_coef <- correlation(mydata, myvariables, cor_variables))

# 2 example
cor_variables <- c("escs","hisei", "scie")
(cor_coef <- correlation(mydata, myvariables, cor_variables))

# 3 example
cor_variables <- c("escs","hisei", "scie")
group <- c("st004d01t")
(cor_coef <- correlation(mydata, myvariables, cor_variables, group = group))

# 4 example
cor_variables <- c("escs", "pv1scie")
method <- "Spearman"
(cor_coef <- correlation(mydata, myvariables, cor_variables, method = method))


#************************************
# Regression
#************************************

# Linear Regression
# 1 example
depended <- "scie" 
independed <- c("st011q12ta", "st004d01t")
package <- 1 #1 - BIFIEsurvey, 2 - EdSurvey, 3 - intsvy
reg_equation <- line_regression(mydata, myvariables, depended, independed, package)
summary(reg_equation)

# 2 example
depended <- "scie" 
independed <- c("st011q12ta", "st004d01t")
package <- 2 #1 - BIFIEsurvey, 2 - EdSurvey, 3 - intsvy
reg_equation <- line_regression(mydata, myvariables, depended, independed, package)
summary(reg_equation)

# 3 example
depended <- "scie" 
independed <- c("st011q12ta", "st004d01t")
package <- 3 #1 - BIFIEsurvey, 2 - EdSurvey, 3 - intsvy
reg_equation <- line_regression(mydata, myvariables, depended, independed, package)
summary(reg_equation)

# Multi Linear Regression
depended <- c("scie","read") 
independed <- c("st004d01t")
reg_equation_multi <- multi_regression(mydata, depended, independed)
summary(reg_equation_multi)

# Logistic Regression
depended <- "scie"
independed <- c("st011q12ta", "st004d01t")
(reg_log_equation <- log_regression(mydata, myvariables, depended, independed))


#************************************
# Multilevel
#************************************

#=* TIMSS *=#
## NULL model
model_null <- ilsa_multilevel(data=my_timss_data, depended="mmat", cluster = "idschool", weightVar=c("stclwgt","schwgt"), weightTransformation=FALSE)
summary(model_null)

## Random intercepts model with level 1 predictor
# The asdhedup variable is categorical. If no changes are made, the multilevel model will be built for each category separately
model_ancova1 <- ilsa_multilevel(data=my_timss_data, depended="mmat", wl_independed=c("asdhedup"),cluster = "idschool", weightVar=c("stclwgt","schwgt"), weightTransformation=FALSE)
summary(model_ancova1)

# Numeric recoding of a categorical variable
change <- c("SOME PRIMARY, LOWER SECONDARY OR NO SCHOOL", 5, "LOWER SECONDARY", 4, "UPPER SECONDARY", 3, "POST-SECONDARY BUT NOT UNIVERSITY", 2,"UNIVERSITY OR HIGHER",1)
my_timss_data <- replace_value(mydata = my_timss_data, oldname = "asdhedup", change = change)
# model
model_ancova2 <- ilsa_multilevel(data=my_timss_data, depended="mmat", wl_independed=c("asdhedup"),cluster = "idschool", weightVar=c("stclwgt","schwgt"), weightTransformation=FALSE)
summary(model_ancova2)

## Random intercepts model with level 1 and level 2 predictors – compositional effect
change <- c("DID NOT ATTEND", 0, "1 YEAR OR LESS", 1, "2 YEARS", 2, "3 YEARS OR MORE", 3)
my_timss_data <- replace_value(mydata = my_timss_data, oldname = "asdhaps", change = change)
# at the second level, the data must be prepared on the basis of the group average
asdhaps2l <- aggregate(asdhaps~idschool,my_timss_data,mean)
colnames(asdhaps2l) <- c("idschool","asdhaps2l")
my_timss_data <- merge(x = my_timss_data,y = asdhaps2l, by = "idschool", all=TRUE)
#model
model_random <- ilsa_multilevel(data=my_timss_data, depended="mmat", wl_independed=c("asdhedup"), bl_independed=c("asdhaps2l"), cluster = "idschool", weightVar=c("stclwgt","schwgt"), weightTransformation=FALSE)
summary(model_random)

## Models comparison
models.summary(list(model_null,model_ancova2,model_random))
