summary.EFECTASMultilevel <- function (object, document = "multilevel.xlsx", ...) 
{
    options(scipen=999)
    object$vars <- object$varsmatSum
    wb <-  createWorkbook()
    object$message <- excel.output(list(object), document=document, wb=wb)
    saveWorkbook(wb, document, overwrite = TRUE)
    class(object) <- "summary.EFECTASMultilevel"
    return(object)
}

print.summary.EFECTASMultilevel <- function(x, digits = max(3, getOption("digits") - 3), nsmall=2, ...) {
    options(knitr.kable.NA = "")
    cat("\n")
    cat("MODEL INFO:\n")
    cat("-----------\n\n")
    if(x$npv>1){
        cat(paste0("Plausible Values: ", x$npv))
        cat("\n\n")
    }
    cat("Number of Groups:")
    print(knitr::kable(x$groupSum,"simple", digits=2), row.names=FALSE, ...)
    cat("\n")
    cat(paste0("Formula: ", paste(deparse(x$formula), collapse=""),"\n"))
    if (!is.na(x$ICC)) {
        cat(paste0("\nIntraclass Correlation= ", format(x$ICC, nsmall=3, digits=3)),"\n")
    }
    
    cat("\n")
    cat("MODEL FIT INFORMATION:\n")
    cat("----------------------")
    if(x$npv>1){
        print(knitr::kable(x$FI,"simple", digits=2), row.names=FALSE,...)
    } else {
        print(knitr::kable(x$FI,"simple", digits=2, col.names = NULL), row.names=FALSE,...)
    }
    cat("\n")
    cat("MODEL RESULTS:\n")
    cat("--------------\n")
    cat("Fixed Effects:")
    print(knitr::kable(x$coef,"simple",digits=2), ...)
    cat("\n")
    cat("Random Effects:")
    vars <- x$vars
    cori <- 1
    corvi <- paste0("Corr",cori)
    while(corvi %in% colnames(vars)) {
        vars[[corvi]] <- as.character(round(vars[[corvi]], 2))
        cori <- cori + 1
        corvi <- paste0("Corr",cori)
    }
    print(knitr::kable(vars,"simple", digits=2), row.names=FALSE, ...)
    cat("\n")
    cat(x$message)
}

excel.output <- function(models, document=NULL, wb=NULL, ...){
    #models <- list(mlmmano,mlmmano1,mlmmano2)
    #models <- list(mlmmano,mlmmano,mlmmano)
    #models <- list(mlmmano1)
    #ob <- models[[1]]
    # calculates how many models need to be output
    model_number <- length(models)
    model_name <- NULL
    # set sheet names (1 model or all models)
    if(model_number == 1){sheet_name <- "Model"} else {sheet_name <- "All Models"}
    # fixed coefficients first line
    f_coef <- list()
    # fixed coefficient other lines
    a_coef <- list()
    coef_row_names <- list()
    #max_coef <- NULL
    #max_var <- NULL
    # rand coefficients first and last line
    w_var <- list()
    b_var <- list()
    # rand coefficients other lines
    a_var <- list()
    var_row_names <- list()
    # fit indices
    l_aic <- list()
    l_bic <- list()
    
    for(i in 1:model_number){
        #i <- 3
        if(nrow(models[[i]]$coef)==1){
            model_name[i] <- "Model 1"
            f_coef[[i]] <- models[[i]]$coef
            a_coef[[i]] <- NULL
            coef_row_names[[i]] <- NULL
        } else {
            model_name[i] <- paste("Model",i)
            f_coef[[i]] <- models[[i]]$coef[1,,drop=FALSE]
            a_coef[[i]] <- models[[i]]$coef[2:nrow(models[[i]]$coef),,drop=FALSE]
            coef_row_names[[i]] <- rownames(a_coef[[i]])
        }
        # rand coefficients only need columns
        var_new <- models[[i]]$varsmatSum[,c(3:5,7:8)]
        var_new[nrow(var_new),1] <- "Residual"
        rownames(var_new) <- var_new$Name
        var_new$Name <- NULL
        if(nrow(var_new)==2){
            w_var[[i]] <- var_new[nrow(var_new),,drop=FALSE]
            b_var[[i]] <- var_new[1,,drop=FALSE]
            a_var[[i]] <- NULL
            var_row_names[[i]] <- NULL
        } else {
            w_var[[i]] <- var_new[nrow(var_new),,drop=FALSE]
            b_var[[i]] <- var_new[1,,drop=FALSE]
            a_var[[i]] <- var_new[2:(nrow(var_new)-1),,drop=FALSE]
            var_row_names[[i]] <- rownames(a_var[[i]])
        }
        l_aic[[i]] <- models[[i]]$FI[2,1]
        l_bic[[i]] <- models[[i]]$FI[3,1]
    }
    crnall <- unique(Reduce(c,coef_row_names))
    crnall <- sort(crnall)
    crnall <- as.data.frame(crnall)
    vrnall <- unique(Reduce(c,var_row_names))
    vrnall <- sort(vrnall)
    vrnall <- as.data.frame(vrnall)
    min_aic <- which.min(l_aic)
    min_bic <- which.min(l_bic)
    
    
    # rows names
    if(model_number==1){
        output_table_names <- c("Observation","Intercept","","Within level",
                                if(length(coef_row_names)>0){coef_row_names[[1]]},
                                "Variance/Residual variance","","Between level",
                                if(length(var_row_names)>0){var_row_names[[1]]},
                                "Variance/Residual variance","",
                                "AIC","BIC","","ICC")
        end_col <- 5
    } else {
        output_table_names <- c("Observation","Intercept","","Within level",
                                if(!is.null(crnall)){crnall$crnall},
                                "Variance/Residual variance","","Between level",
                                if(!is.null(vrnall)){vrnall$vrnall},
                                "Variance/Residual variance","",
                                "AIC","BIC","","ICC")
        end_col <- model_number * 4 + 1
    }
    
    
    # Style format
    first_header_row <- createStyle(fgFill = "#72AFF3", 
                                    textDecoration = "bold",
                                    border = c("top", "bottom", "left", "right"), 
                                    borderColour = "#000000")
    second_header_row <- createStyle(fgFill = "#BBE0FF", 
                                     border = c("top", "bottom", "left", "right"), 
                                     borderColour = "#000000")
    level_row <- createStyle(fgFill="#72AFF3",
                             textDecoration = "italic",
                             border = c("top", "bottom", "left", "right"), 
                             borderColour = "#000000")
    fit_indices_row <- createStyle(numFmt = "0.00",
                                   fgFill="#C8F2FF",
                                   border = c("top", "bottom", "left", "right"), 
                                   borderColour = "#000000")
    obser_row <- createStyle(border = c("top", "bottom", "left", "right"), 
                             borderColour = "#000000")
    options(scipen=999)
    decimals_style <- createStyle(numFmt = "0.00",
                                  border = c("top", "bottom", "left", "right"), 
                                  borderColour = "#000000")
    
    # count columns
    start_col <- 2
    output_cols <- c((start_col-1):end_col)
    decimal_cols <- c(start_col:end_col)
    fit_rows <- 4
    all_rows <- length(output_table_names) - fit_rows
    addWorksheet(wb, sheet_name)
    setColWidths(wb, sheet_name, cols = output_cols, widths = "auto")
    
    # adding style
    for(i in seq(2, end_col, by=4)){
        ## header
        mergeCells(wb, sheet_name, cols = i:(i+3), rows = 1)
        addStyle(wb, sheet = sheet_name, 
                 style = first_header_row, cols = i:(i+3), 
                 rows = 1, gridExpand = TRUE)
    }
    ## second header
    addStyle(wb, sheet = sheet_name, 
             style = second_header_row, cols = decimal_cols, 
             rows = 2, gridExpand = TRUE)
    ## model results
    addStyle(wb, sheet = sheet_name, 
             style = obser_row, cols = decimal_cols, 
             rows = 3, gridExpand = TRUE)
    addStyle(wb, sheet = sheet_name, 
             style = decimals_style, cols = decimal_cols, 
             rows = 4:(2+all_rows), gridExpand = TRUE)
    ## row names
    addStyle(wb, sheet = sheet_name, 
             style = second_header_row, cols = 1, 
             rows = 3:(all_rows+2), gridExpand = TRUE)
    addStyle(wb, sheet = sheet_name, 
             style = fit_indices_row, cols = output_cols, 
             rows = (all_rows+3):(all_rows + 2 + fit_rows), gridExpand = TRUE)
    
    for(i in 1:model_number){
        #i <- 2
        # model name
        writeData(wb, sheet_name, model_name[i] ,startCol = start_col, startRow = 1,colNames = FALSE ,rowNames = FALSE)
        # model results
        header <- c("Estimate", "Std. Error", "t value", "P-value")
        writeData(wb, sheet_name, t(header),startCol = start_col, startRow = 2,colNames = FALSE)
        row_number <- 3
        obser <- models[[i]]$nUsed
        writeData(wb, sheet_name, obser,startCol = start_col, startRow = row_number)
        row_number <- row_number + 1
        writeData(wb, sheet_name, f_coef[[i]],startCol = start_col, startRow = row_number,colNames = FALSE ,rowNames = FALSE)
        row_number <- row_number + 3
        if(i==model_number){
            addStyle(wb, sheet = sheet_name, 
                     style = level_row, cols = output_cols, 
                     rows = row_number-1, gridExpand = TRUE)
        }
        if(length(a_coef)>0){
            temp_coef <- as.data.frame(a_coef[[i]])
            temp_coef$names <- rownames(temp_coef)
            out_coef <- merge(crnall,temp_coef,by.x = "crnall", by.y = "names",all.x=TRUE)
            out_coef$crnall <- NULL
            out_coef[is.na(out_coef)] <- "" 
            out_coef <- lapply(out_coef,as.numeric)
            writeData(wb, sheet_name, out_coef,startCol = start_col, startRow = row_number,colNames = FALSE ,rowNames = FALSE)
            rn <- nrow(crnall)
        } else {
            rn <- 0
        }
        row_number <- row_number + rn
        writeData(wb, sheet_name, w_var[[i]],startCol = start_col, startRow = row_number,colNames = FALSE ,rowNames = FALSE)
        row_number <- row_number + 3
        if(i==model_number){
            addStyle(wb, sheet = sheet_name, 
                     style = level_row, cols = output_cols, 
                     rows = row_number-1, gridExpand = TRUE)
        }
        if(length(a_var)>0){
            temp_var <- as.data.frame(a_var[[i]])
            temp_var$names <- rownames(temp_var)
            out_var <- merge(vrnall,temp_var,by.x = "vrnall", by.y = "names",all.x=TRUE)
            out_var$vrnall <- NULL
            out_var[is.na(out_var)] <- "" 
            out_var <- lapply(out_var,as.numeric)
            writeData(wb, sheet_name, out_var,startCol = start_col, startRow = row_number,colNames = FALSE ,rowNames = FALSE)
            rnn <- nrow(vrnall)
        } else {
            rnn <- 0
        }
        row_number <- row_number + rnn
        writeData(wb, sheet_name, b_var[[i]],startCol = start_col, startRow = row_number,colNames = FALSE ,rowNames = FALSE)
        row_number <- row_number + 2
        writeData(wb, sheet_name, l_aic[[i]],startCol = start_col, startRow = row_number)
        row_number <- row_number + 1
        writeData(wb, sheet_name, l_bic[[i]],startCol = start_col, startRow = row_number)
        row_number <- row_number + 2
        l_icc <- models[[i]]$ICC
        writeData(wb, sheet_name, l_icc,startCol = start_col, startRow = row_number)
        start_col <- start_col + 4
    }
    writeData(wb, sheet_name, output_table_names,startCol = 1, startRow = 3)
    if(model_number > 1){
        writeData(wb, sheet_name, paste("The best model according to AIC is", model_name[min_aic]),startCol = 1, startRow = row_number+3)
        writeData(wb, sheet_name, paste("The best model according to BIC is", model_name[min_bic]),startCol = 1, startRow = row_number+4)
    }
    output_message <- paste("Model data saved in a document ", document)
    return(output_message)
}


models.summary <- function(models, document="MLM_compare.xlsx", ...){
    # calculate how many models 
    model_number <- length(models)
    if(model_number == 1){
        stop("Only one model cannot be used to compare models. Please specify at least two models. ")
    } else {
        wb <-  createWorkbook()
        my_output <- excel.output(models, document=document, wb=wb)
        saveWorkbook(wb, document, overwrite = TRUE)
    }
    return(my_output)
}
