# ---------------------------------------------------------------------------------------------------------------------------
# Data Formation
# ---------------------------------------------------------------------------------------------------------------------------

## Preraring PISA, TIMSS and PIRLS data

data_form <- function(path_root, 
                      ILSA=c("PISA","TIMSS","PIRLS"), 
                      years=c(2015, 2016, 2018, 2019),
                      mycountry, 
                      myvariables,
                      gradeLvl = 4){
    validILSA <- c("PISA","TIMSS","PIRLS")
    if (!ILSA %in% validILSA){
        stop(paste("This tool does not process",ILSA,"data\n"))
    } else {
        if (ILSA == "PISA"){
            validyear <- c(2015, 2018)
            if (!years %in% validyear){
                stop(paste("This tool does not process",ILSA,years,"data. Valid years: 2015 and 2018.\n"))
            } else {
                new_path_root <- paste0(path_root, "PISA/",years)
                if (mycountry == "all"){ 
                    # Assigns all country codes. Not recommended
                    mycountry <- c("ALB", "ARE", "AUS", "AUT", "BEL", "BGR", "BRA", "CAN", "CHE", "CHL",
                                   "COL", "CRI", "CZE", "DEU", "DNK", "DOM", "DZA", "ESP", "EST", "FIN",
                                   "FRA", "GBR", "GEO", "GRC", "HKG", "HRV", "HUN", "IDN", "IRL", "ISL",
                                   "ISR", "ITA", "JOR", "JPN", "KOR", "KSV", "LBN", "LTU", "LUX", "LVA",
                                   "MAC", "MDA", "MEX", "MKD", "MLT", "MNE", "NLD", "NOR", "NZL", "PER",
                                   "POL", "PRT", "QAR", "QAT", "QCH",  "QES", "QUC", "QUD", "QUE", "ROU",
                                   "RUS", "SGP", "SVK", "SVN", "SWE", "TAP", "THA", "TTO", "TUN", "TUR",
                                   "URY", "USA", "VNM")
                }
                Pisa.data <- readPISA(path = new_path_root, database = "INT", countries = mycountry)
                must_coll <- c("cntryid", "cnt", "cntschid", "cntstuid", "w_fstuwt", "w_fsturwt1", "w_fsturwt2", "w_fsturwt3", 
                               "w_fsturwt4", "w_fsturwt5", "w_fsturwt6", "w_fsturwt7", "w_fsturwt8", "w_fsturwt9", 
                               "w_fsturwt10", "w_fsturwt11", "w_fsturwt12", "w_fsturwt13", "w_fsturwt14", "w_fsturwt15", 
                               "w_fsturwt16", "w_fsturwt17", "w_fsturwt18", "w_fsturwt19", "w_fsturwt20", "w_fsturwt21", 
                               "w_fsturwt22", "w_fsturwt23", "w_fsturwt24", "w_fsturwt25", "w_fsturwt26", "w_fsturwt27", 
                               "w_fsturwt28", "w_fsturwt29", "w_fsturwt30", "w_fsturwt31", "w_fsturwt32", "w_fsturwt33", 
                               "w_fsturwt34", "w_fsturwt35", "w_fsturwt36", "w_fsturwt37", "w_fsturwt38", "w_fsturwt39", 
                               "w_fsturwt40", "w_fsturwt41", "w_fsturwt42", "w_fsturwt43", "w_fsturwt44", "w_fsturwt45", 
                               "w_fsturwt46", "w_fsturwt47", "w_fsturwt48", "w_fsturwt49", "w_fsturwt50", "w_fsturwt51", 
                               "w_fsturwt52", "w_fsturwt53", "w_fsturwt54", "w_fsturwt55", "w_fsturwt56", "w_fsturwt57", 
                               "w_fsturwt58", "w_fsturwt59", "w_fsturwt60", "w_fsturwt61", "w_fsturwt62", "w_fsturwt63", 
                               "w_fsturwt64", "w_fsturwt65", "w_fsturwt66", "w_fsturwt67", "w_fsturwt68", "w_fsturwt69", 
                               "w_fsturwt70", "w_fsturwt71", "w_fsturwt72", "w_fsturwt73", "w_fsturwt74", "w_fsturwt75", 
                               "w_fsturwt76", "w_fsturwt77", "w_fsturwt78", "w_fsturwt79", "w_fsturwt80", "w_fstuwt_sch_sum")
                all_coll <- c(myvariables, must_coll)
                data.file <- getData(Pisa.data, varnames = all_coll,
                                     addAttributes = TRUE, omittedLevels = FALSE, defaultConditions = FALSE)
            }
        } else if (ILSA == "TIMSS"){
            validyear <- c(2015, 2019)
            if (!years %in% validyear){
                stop(paste("This tool does not process",ILSA,years,"data. Valid years: 2015 and 2019.\n"))
            } else {
                new_path_root <- paste0(path_root, "TIMSS/",years)
                if (mycountry == "all"){ 
                    # Assigns all country codes. Not recommended
                    mycountry <- c("AAD",	"ADU",	"ALB",	"ARE",	"ARM",	"AUS",	"AUT",	"AZE",	"BFL",	"BGR",
                                   "BHR",	"BIH",	"CAN",	"CHL",	"CYP",	"COT",	"CQU",	"CZE",	"DEU",	"DNK",
                                   "EGY",	"EMA",	"ENG",	"ESP",	"FIN",	"FRA",	"GEO",	"HKG",	"HRV",	"HUN",
                                   "IRL",	"IRN",	"ISR",	"ITA",	"JOR",	"JPN",	"KAZ",	"KOR",	"KWT",	"LBN",
                                   "LTU",	"LVA",	"MAR",	"MYS",	"MKD",	"MLT",	"MNE",	"NIR",	"NLD",	"NOR",
                                   "NZL",	"OMN",	"PAK",	"PHL",	"POL",	"PRT",	"QAT",	"RMO",	"ROM",	"RUS",
                                   "SAU",	"SGP",	"SRB",	"SVK",	"SWE",	"TUR",	"TWN",	"USA",	"XKX",	"ZAF",
                                   "ZGT",	"ZWC")
                }
                Timss.data <- readTIMSS(path = new_path_root, countries = mycountry, gradeLvl = gradeLvl)
                must_coll <- c("idcntry", "idschool", "idclass", "idstud", "totwgt", "jkzone", "jkrep", 
                               "wgtadj1","wgtadj2","wgtadj3","wgtfac1","wgtfac2","wgtfac3")
                all_coll <- c(myvariables, must_coll)
                data.file <- getData(Timss.data, varnames = all_coll,
                                     addAttributes = TRUE, omittedLevels = FALSE, defaultConditions = FALSE)
                data.file$schwgt <- data.file$wgtadj1 * data.file$wgtfac1
                data.file$clwgt <- data.file$wgtadj2 * data.file$wgtfac2
                data.file$stwgt <- data.file$wgtadj3 * data.file$wgtfac3
                data.file$stclwgt <- data.file$schwgt * data.file$clwgt
            }
        } else if (ILSA == "PIRLS"){
            validyear <- c(2016)
            if (!years %in% validyear){
                stop(paste("This tool does not process",ILSA,years,"data. Valid years: 2016.\n"))
            } else {
                new_path_root <- paste0(path_root, "PIRLS/",years)
                if (mycountry == "all"){ 
                    # Assigns all country codes. Not recommended
                    mycountry <- c("AAD",	"ABA",	"ADU",	"ARE",	"AUS",	"AUT",	"AZE",	"BFL",	"BFR",	"BGR",
                                   "BHR",	"CAN",	"CHL",	"COT",	"CQU",	"CZE",	"DEU",	"DN3",	"DNK",	"EAN",
                                   "EGY",	"EMA",	"ENG",	"ESP",	"FIN",	"FRA",	"GEO",	"HKG",	"HUN",	"IRL",
                                   "IRN",	"ISR",	"ITA",	"KAZ",	"KWT",	"LTU",	"LVA",	"MAC",	"MAR",	"MLT",
                                   "NIR",	"NLD",	"NO4",	"NOR",	"NZL",	"OMN",	"POL",	"PRT",	"QAT",	"RMO",
                                   "RUS",	"SAU",	"SGP",	"SVK",	"SVN",	"SWE",	"TTO",	"TWN",	"USA",	"ZA5",
                                   "ZAF")
                }
                Pirls.data <- readPIRLS(path = new_path_root, countries = mycountry)
                must_coll <- c("idcntry", "idschool", "idclass", "idstud", "totwgt", "jkzone", "jkrep", 
                               "wgtadj1","wgtadj2","wgtadj3","wgtfac1","wgtfac2","wgtfac3")
                all_coll <- c(myvariables, must_coll)
                data.file <- getData(Pirls.data, varnames = all_coll,
                                     addAttributes = TRUE, omittedLevels = FALSE, defaultConditions = FALSE)
                data.file$schwgt <- data.file$wgtadj1 * data.file$wgtfac1
                data.file$clwgt <- data.file$wgtadj2 * data.file$wgtfac2
                data.file$stwgt <- data.file$wgtadj3 * data.file$wgtfac3
                data.file$stclwgt <- data.file$schwgt * data.file$clwgt
            }
        }
    }
    return(data.file)
}

# help to clear data like OECD
replace_value <-  function(mydata, oldname, newname = NULL, change){
    ilgis <- length(change)
    if(is.null(newname)){
        newname <- "naujas"
    }
    for(i in 1:ilgis){
        if (i %% 2 == 1){
            mydata[[newname]][mydata[[oldname]] == change[i]] <- as.numeric(change[i+1])
        }
    }
    if(newname == "naujas"){
        mydata[[oldname]] <- mydata[[newname]]
        mydata[[newname]] <- NULL
        newname <- NULL
    }
    if(!is.null(newname)){
        if(oldname != newname){
            mydata[,oldname] <- NULL
        } 
    }
    return(mydata)
}

replace_value2 <-  function(mydata2, oldname, newname = NULL, change, text=FALSE){
    #mydata2 <- mydata1
    #oldname <- "st004d01t"
    #newname <- "gender"
    #change <- c("FEMALE", 1, "MALE", 0)
    ilgis <- length(change)
    if(is.null(newname)){
        newname <- "naujas"
    }
    if(text == FALSE){
        for(i in 1:ilgis){
            # i <- 5
            if (i %% 2 == 1){
                # print(change[i])
                #  print(as.numeric(change[i+1]))
                mydata2[[newname]][mydata2[[oldname]] == change[i]] <- as.numeric(change[i+1])
            }
        }
    } else {
        for(i in 1:ilgis){
            # i <- 5
            if (i %% 2 == 1){
                # print(change[i])
                #  print(as.numeric(change[i+1]))
                mydata2[[newname]][mydata2[[oldname]] == change[i]] <- change[i+1]
            }
        }
    }
    if(newname == "naujas"){
        mydata2[[oldname]] <- mydata2[[newname]]
        mydata2[[newname]] <- NULL
        newname <- NULL
    }
    if(!is.null(newname)){
        if(oldname != newname){
            mydata2[,oldname] <- NULL
        } 
    }
    return(mydata2)
}

# weiht normalization
normalization_weight <- function(mydata){
    mydata[order(mydata$cnt),]
    suma <- sum(mydata[,"w_fstuwt"])
    N <- nrow(mydata)
    mydata$w_fsturwt0 <- mydata$w_fstuwt
    stulpeliai <- c("w_fsturwt0","w_fsturwt1", "w_fsturwt2", "w_fsturwt3", 
                    "w_fsturwt4", "w_fsturwt5", "w_fsturwt6", "w_fsturwt7", "w_fsturwt8", "w_fsturwt9", 
                    "w_fsturwt10", "w_fsturwt11", "w_fsturwt12", "w_fsturwt13", "w_fsturwt14", "w_fsturwt15", 
                    "w_fsturwt16", "w_fsturwt17", "w_fsturwt18", "w_fsturwt19", "w_fsturwt20", "w_fsturwt21", 
                    "w_fsturwt22", "w_fsturwt23", "w_fsturwt24", "w_fsturwt25", "w_fsturwt26", "w_fsturwt27", 
                    "w_fsturwt28", "w_fsturwt29", "w_fsturwt30", "w_fsturwt31", "w_fsturwt32", "w_fsturwt33", 
                    "w_fsturwt34", "w_fsturwt35", "w_fsturwt36", "w_fsturwt37", "w_fsturwt38", "w_fsturwt39", 
                    "w_fsturwt40", "w_fsturwt41", "w_fsturwt42", "w_fsturwt43", "w_fsturwt44", "w_fsturwt45", 
                    "w_fsturwt46", "w_fsturwt47", "w_fsturwt48", "w_fsturwt49", "w_fsturwt50", "w_fsturwt51", 
                    "w_fsturwt52", "w_fsturwt53", "w_fsturwt54", "w_fsturwt55", "w_fsturwt56", "w_fsturwt57", 
                    "w_fsturwt58", "w_fsturwt59", "w_fsturwt60", "w_fsturwt61", "w_fsturwt62", "w_fsturwt63", 
                    "w_fsturwt64", "w_fsturwt65", "w_fsturwt66", "w_fsturwt67", "w_fsturwt68", "w_fsturwt69", 
                    "w_fsturwt70", "w_fsturwt71", "w_fsturwt72", "w_fsturwt73", "w_fsturwt74", "w_fsturwt75", 
                    "w_fsturwt76", "w_fsturwt77", "w_fsturwt78", "w_fsturwt79", "w_fsturwt80")
    mydata[,stulpeliai] <- (mydata[,stulpeliai]/suma)*N
    #mydata$w_fstuwt <- NULL
    mydata$nbre <- N
    mydata$somwgt <- suma
    return(mydata)
}

# BIFI_data_pv function for BIFIobject formation when plausible values are combined
BIFI_data_pv <- function(package_data, myvariables){
    data <- package_data
    # user used plausible values
    all_pv_names <- c("math", "read", "scie", "scep", "sced", "scid", "skco", "skpe", "ssph", "ssli", "sses", "flit", "clps")
    my_pv_names <- intersect(myvariables,all_pv_names)
    pv_vars <- my_pv_names
    # create 10 datasets including different sets of plausible values
    dfr <- NULL
    VV <- length(pv_vars)
    Nimp <- 10 # number of plausible values
    for (vv in 1:VV){
        vv1 <- pv_vars[vv]
        ind.vv1 <- which( colnames(data) %in% paste0("pv", 1:Nimp, vv1) )
        dfr2 <- data.frame( "variable"=paste0("PV", vv1), "var_index"=vv,
                            "data_index"=ind.vv1, "impdata_index"=1:Nimp )
        dfr <- rbind( dfr, dfr2 )
    }
    sel_ind <- setdiff( 1:( ncol(data) ), dfr$data_index )
    data0 <- data[, sel_ind ]
    V0 <- ncol(data0)
    newvars <- seq( V0+1, V0+VV )
    datalist <- as.list( 1:Nimp )
    for (ii in 1:Nimp ){
        dat1 <- data.frame( data0, data[, dfr[ dfr$impdata_index==ii, "data_index" ]])
        colnames(dat1)[ newvars ] <- pv_vars
        datalist[[ii]] <- dat1
    }
    # dataset with replicate weights
    datarep <- data[, grep( "w_fsturwt", colnames(data) ) ]
    RR <- ncol(datarep) # number of replicate weights
    # create BIFIE object
    bifieobj <- BIFIEsurvey::BIFIE.data( datalist, wgt=data[, "w_fstuwt"],
                                         wgtrep=datarep, fayfac=1 / RR / ( 1 - .5 )^2 )
    # For PISA: RR=80 and therefore fayfac=1/20=.05
    return(bifieobj)
}

# BIFI_data function for BIFIobject formation without combining plausible values
BIFI_data <- function(package_data){
    data <- package_data
    # dataset with replicate weights
    datarep <- data[, grep( "w_fsturwt", colnames(data) ) ]
    RR <- ncol(datarep) # number of replicate weights
    # create BIFIE object
    bifieobj <- BIFIEsurvey::BIFIE.data( data, wgt=data[, "w_fstuwt"],
                                         wgtrep=datarep, fayfac=1 / RR / ( 1 - .5 )^2)
    # For PISA: RR=80 and therefore fayfac=1/20=.05
    return(bifieobj)
}
