# ---------------------------------------------------------------------------------------------------------------------------
# Descriptive statistic
# ---------------------------------------------------------------------------------------------------------------------------


# Frequency table, percentage, missing values
frequency_table <- function(mydata, myvariable, variables, mygroup = NULL, missing_values = FALSE, uniq = FALSE){
    if(missing_values == FALSE){
        if(any(grepl("pv",variables))== TRUE){
            bdat <- BIFI_data(mydata)
        } else {
            bdat <- BIFI_data_pv(mydata, myvariable)
        }
        if(uniq == FALSE){
            lentele <- BIFIE.freq(BIFIEobj = bdat, vars = variables, group = mygroup, se = FALSE) # BIFIEsurvey
            lentele[["stat"]][["perc"]] <- lentele[["stat"]][["perc"]]*100
        } else {
            lentele <- BIFIE.by(BIFIEobj = bdat, vars = variables, userfct = userfct, userparnames = paste0(variables), 
                                group = mygroup, se = FALSE) # BIFIEsurvey
        }
        lentele <- lentele[["stat"]]
    } else {
        if(length(variables) == 1){
            lentele <- summary2(data = Pisa.data,variable = variables, weightVar = "w_fstuwt") # EdSurvey
        } else {
            stop('Missing values can be calculated only for one variable') 
        }
    }
    return(lentele)
}

# Function for number of unique entries. BIFIEsurvey
userfct <- function(X,w){
    pars <- apply( X, 2, FUN = function(vv){
        length( unique(vv))  
    })
    return(pars)
}

# Min, Max, Average, Standard deviation, Median, Quartile for ordinal variables. EdSurvey
descriptive <- function(variables){
    table <- summary2(data = Pisa.data, variable = variables, weightVar = "w_fstuwt")
    return(table)
}

# Percentiles EdSurvey
tile <- function(variables, percent){
    percentiliai <- percentile(variable = variables, percentiles = percent, data = Pisa.data, weightVar = "w_fstuwt")
    return(percentiliai)
}

# Benchmarks only for all plausible values intsvy
ben_marks <- function(mydata, variable, bench){
    colnames(mydata) <- toupper(colnames(mydata))
    variable <- toupper(variable)
    zymes <- pisa2015.ben.pv(pvlabel = variable, by = "CNT", cutoff = bench, data = mydata)
    colnames(mydata) <- tolower(colnames(mydata))
    return(zymes)
}
