ilsa_multilevel <- function (data, 
                             depended,
                             wl_independed = NULL,
                             bl_independed = NULL,
                             cluster,
                             weightVars = NULL, 
                             weightTransformation = TRUE, 
                             recode = NULL, 
                             defaultConditions = TRUE, 
                             tolerance = 0.01, 
                             nQuad = NULL, 
                             verbose = 0, 
                             family = NULL, 
                             centerGroup = NULL, 
                             centerGrand = NULL, 
                             fast = FALSE,
                             ...) {
    call <- match.call()
    call0 <- call
    if (!missing(nQuad) & is.null(family)) {
        warning(paste0("The ", sQuote("nQuad"), " argument is depreciated for linear models."))
    }
    if (!missing(tolerance) & is.null(family)) {
        warning(paste0("The ", sQuote("tolerance"), " argument is depreciated."))
    }
    if (!missing(fast)) {
        warning(paste0("The ", sQuote("fast"), " argument is depreciated."))
    }
    if (!missing(family)) {
        stop(paste0("The ", dQuote("family"), " argument is depreciated; plase use the ", 
                    dQuote("WeMix"), " package's ", dQuote("mix"), " function direclty for binomial models."))
    }
    # formula 
    if(is.null(wl_independed) && is.null(bl_independed)){
        dep_in <- paste(depended, 1, sep=" ~ ")
        f_class <-  paste("(",paste(1,cluster,sep = "|"),")")
        mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
    } else if (is.null(bl_independed)){
        dep_in <- (paste(depended, paste(wl_independed, collapse=" + "), sep=" ~ "))
        f_class <-  paste("(",paste(1,cluster,sep = "|"),")")
        mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
    } else {
        dep_in <- (paste(depended, paste(wl_independed, collapse=" + "), sep=" ~ "))
        f_class <-  paste("(",paste(paste(bl_independed, collapse=" + "),cluster,sep = "|"),")")
        mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
    }
    
    formula <- mula
    formula0 <- formula
    EdSurvey:::checkDataClass(data, c("edsurvey.data.frame", "light.edsurvey.data.frame"))
    survey <- getAttributes(data, "survey")
    if (is.null(weightVars)) {
        if (survey == "PISA") {
            weightVars <- c("w_fstuwt", "w_fschwt")
        }
        else if (survey %in% c("TIMSS", "PIRLS")) {
            weightVars <- c("totwgt", "schwgt")
        }
        else {
            stop("mixed.sdf currently only supports automated weights for PISA, TIMSS, and TIMSS Advanced. If you use another survey, please specify your own weights. ")
        }
        call$weightVars <- weightVars
    }
    
    if (!inherits(formula, "formula")) {
        stop(paste0(sQuote("formula"), " argument must be of class formula."))
    }
    
    zeroLengthLHS <- attr(terms(formula), "response") == 0
    if (zeroLengthLHS) {
        yvar <- attributes(getAttributes(data, "pvvars"))$default
        formula <- update(formula, new = substitute(yvar ~ ., 
                                                    list(yvar = as.name(yvar))))
    }
    else {
        yvar <- all.vars(formula[[2]])
    }
    pv <- hasPlausibleValue(yvar, data)
    yvars <- yvar
    linkingError <- "NAEP" %in% getAttributes(data, "survey") & 
        any(grepl("_linking", yvars, fixed = TRUE))
    if (linkingError) {
        stop("mixed.sdf does not support estimation with linking error.")
    }
    if (pv) {
        yvars <- getPlausibleValue(yvar, data)
    }
    
    
    # Get data
    getDataArgs <- list(data = data, 
                        varnames = unique(c(all.vars(formula), weightVars, yvars)), 
                        returnJKreplicates = FALSE, 
                        drop = FALSE, 
                        omittedLevels = FALSE, 
                        recode = recode, 
                        includeNaLabel = TRUE, 
                        dropUnusedLevels = TRUE)
    
    if (!missing(defaultConditions)) {
        getDataArgs <- c(getDataArgs, list(defaultConditions = defaultConditions))
    }
    
    edf <- do.call(getData, getDataArgs)
    rawN <- nrow(edf)
    
    for (wgt in weightVars) {
        if (any(!(!is.na(edf[, wgt]) & edf[, wgt] > 0))) {
            warning("Removing ", sum(!(!is.na(edf[, wgt]) & 
                                           edf[, wgt] > 0)), " rows with 0 or NA weight on ", 
                    dQuote(wgt), " from analysis.")
            edf <- edf[!is.na(edf[, wgt]) & edf[, wgt] > 0, ]
        }
    }
    
    
    pvy <- hasPlausibleValue(yvar, data)
    yvars <- yvar
    lyv <- length(yvars)
    if (any(pvy)) {
        yvars <- getPlausibleValue(yvar, data)
    }
    else {
        edf[, "yvar"] <- as.numeric(eval(formula[[2]], edf))
        formula <- update(formula, new = substitute(yvar ~ ., 
                                                    list(yvar = as.name(yvar))))
        yvars <- "yvar"
    }
    
    yvar0 <- yvars[1]
    if (!is.null(family) && family$family %in% c("binomial")) {
        if (any(pvy)) {
            for (i in 1:length(yvars)) {
                for (yvi in 1:length(pvy)) {
                    if (pvy[yvi]) {
                        edf[, yvar[yvi]] <- edf[, getPlausibleValue(yvar[yvi], 
                                                                    data)[i]]
                    }
                }
                edf[, yvars[i]] <- as.numeric(eval(formula[[2]], 
                                                   edf))
            }
            oneDef <- max(edf[, yvars], na.rm = TRUE)
            for (i in yvars) {
                edf[, i] <- ifelse(edf[, i] %in% oneDef, 1, 0)
            }
        }
        else {
            oneDef <- max(edf[, yvars], na.rm = TRUE)
            edf[, yvar0] <- ifelse(edf$yvar %in% oneDef, 1, 
                                   0)
        }
    }
    
    
    formula <- update(formula, as.formula(paste0(yvar0, " ~ .")))
    
    lformula <- lFormula(formula = formula, data = edf)
    
    unparsedGroupNames <- names(lformula$reTrms$cnms)
    
    groupParser <- function(groupi) {
        all.vars(formula(paste0("~", groupi)))
    }
    
    groupNames <- rev(unique(unlist(lapply(unparsedGroupNames, groupParser))))
    if (length(groupNames) == 0) {
        stop("The formula only indicates one level. Use lm.sdf instead.")
    }
    if (length(weightVars) != length(groupNames) + 1) {
        stop(paste0("The model requires ", length(groupNames) + 
                        1, " weights."))
    }
    
    level <- length(groupNames) + 1
    if (!weightTransformation) {
        for (wi in 1:length(weightVars)) {
            edf[[paste0("pwt", wi)]] <- edf[, weightVars[wi]]
        }
    }
    else {
        if (survey == "PISA") {
            edf$sqw <- edf[, weightVars[1]]^2
            sumsqw <- aggregate(as.formula(paste0("sqw ~ ", groupNames)), data = edf, sum)
            sumw <- aggregate(as.formula(paste0(weightVars[1], "~", groupNames)), data = edf, sum)
            edf$sumsqw <- sapply(edf[, groupNames], function(s) sumsqw$sqw[sumsqw[, groupNames] == s])
            edf$sumw <- sapply(edf[, groupNames], function(s) sumw[sumw[, groupNames] == s, weightVars[1]])
            edf$pwt1 <- edf[, weightVars[1]] * (edf$sumw/edf$sumsqw)
            edf$pwt2 <- edf[, weightVars[2]]
            edf$sqw <- NULL
            edf$sumsqw <- NULL
            edf$sumw <- NULL
        }
        else if (survey %in% c("TIMSS", "PIRLS")) {
            edf$pwt1 <- edf[, weightVars[1]]/edf[, weightVars[2]]
            edf$pwt2 <- edf[, weightVars[2]]
        }
        else {
            warning(paste0("EdSurvey currently does not specify weight transformation rules for ", 
                           survey, ". Raw weights were used for the analysis."))
            edf$pwt1 <- edf[, weightVars[1]]
            edf$pwt2 <- edf[, weightVars[2]]
        }
    }
    
    # Remove omittedLevels from the data.frame edf
    lev <- unlist(getAttributes(data, "omittedLevels"))
    keep <- rep(0, nrow(edf))
    for (i in 1:ncol(edf)) {
        vari <- names(edf)[i]
        keep <- keep + (tolower(edf[, vari]) %in% tolower(lev))
    }
    if (sum(keep > 0) > 0) {
        edf <- edf[keep == 0, , drop = FALSE]
    }
    formula_pv <- formula
    
    summary.WeMixResults <- getFromNamespace("summary.WeMixResults", "WeMix")
    
    if (!pv) {
        # there are no plausible values, just a single outcome
        res <- EdSurvey:::run_mix(nQuad=nQuad, call=call, formula=formula, edf=edf,
                                  verbose=verbose, family=family,center_group=centerGroup,center_grand=centerGrand,
                                  tolerance=tolerance, fast=fast, ...)
        
        #get enrivonment of WeMix likelihood function to later extract covariance from
        env <- environment(res$lnlf)
        # AIC and BIC calculation
        dof <- length(res$coef) + length(res$theta) + 1
        AIC <- -2 * res$lnl + 2 * dof
        nos <- nrow(edf)
        BIC <- -2 * res$lnl + log(nos) * dof
        
        #use summary to extract variances. 
        model_sum <- summary.WeMixResults(res)
        res$se <- c(model_sum$coef[,2] , model_sum$vars[,2])
        names(res$se) <- c(row.names(model_sum$coef), row.names(model_sum$vars))
        res$vars <- model_sum$vars[,1]
        names(res$vars) <- row.names(model_sum$vars)
        FI <- c(res$lnl,AIC,BIC)
        names(FI) <- c("Loglikelihood","AIC","BIC")
        res$FI <- FI
        # Remove some return values
        res$CMODE <- NULL 
        res$CMEAN <- NULL
        res$hessian <- NULL
        res$call <- call0
        res$formula <- mula
        
        varsmat0 <- model_sum$varsmat
        groupSum <- varsmat0[!duplicated(varsmat0$Level), c("Level", "Group")]
        groupSum$Group[groupSum$Level == 1] <- "Obs"
        groupSum$"n size" <- rev(res$ngroups)
        for (i in 1:length(res$wgtStats)) {
            groupSum$"mean wgt"[groupSum$Level == i] <- res$wgtStats[[i]]$mean
            groupSum$"sum wgt"[groupSum$Level == i] <- res$wgtStats[[i]]$sum
        }
        res$groupSum <- groupSum
        
        varsmat0 <- res$varDF
        # put the corred vcov in it
        m <- length(yvars)
        
        varsmat <- varsmat0[is.na(varsmat0$var2), c("level", "grp", "var1", "vcov", "SEvcov")]
        varsmat$st <- sqrt(varsmat$vcov)
        # add variance estimates
        colnames(varsmat) <- c("Level", "Group", "Name", "Variance", "Std. Error", "Std.Dev.")
        res$varsmatSum <- varsmat
        res$VC <- model_sum$cov_mat
    } else { #run the plausible values version 
        #iterate through the plausible values 
        results <- list()
        variances <- list()
        pvi <- 0
        cat(paste("The data you are analysing consists of",length(yvars),"plausible values\n"))
        for (value in yvars) {
            if (verbose > 0) {
                cat(paste0("Estimating mixed model with ", value, " as the outcome."))
            }
            pvi <- pvi + 1
            cat(paste(pvi," plausible value model calculated\n"))
            
            formula_pv <- update(formula_pv, as.formula(paste(value,"~.")))
            model <- withCallingHandlers(EdSurvey:::run_mix(nQuad = nQuad, 
                                                            call = call, formula = formula_pv, edf = edf, 
                                                            verbose = verbose, family = family, center_group = centerGroup, 
                                                            center_grand = centerGrand, tolerance = tolerance, 
                                                            ...), warning = function(w) {
                                                                if (pvi != 1) {
                                                                    invokeRestart("muffleWarning")
                                                                }
                                                                else {
                                                                    message(conditionMessage(w))
                                                                }
                                                            }, message = function(c) {
                                                                if (pvi != 1) {
                                                                    invokeRestart("muffleMessage")
                                                                }
                                                            })
            dof <- length(model$coef) + length(model$theta) + 1
            model$AIC <- -2 * model$lnl + 2 * dof
            nos <- nrow(edf)
            model$BIC <- -2 * model$lnl + log(nos) * dof
            # save results
            results[[value]] <- model
            model_sum <- summary.WeMixResults(model)
            variances[[value]] <- c(model_sum$coef[, "Std. Error"]^2, model_sum$varDF$SEvcov^2)
            if (verbose > 1) {
                print(model_sum)
            }
        }
        res <- results[[1]]
        cat(paste("Summarises the estimated data for the",length(yvars),"models\n"))
        
        varsmat0 <- model_sum$varsmat
        groupSum <- varsmat0[!duplicated(varsmat0$Level), c("Level", "Group")]
        groupSum$Group[groupSum$Level == 1] <- "Obs"
        groupSum$"n size" <- rev(res$ngroups)
        for (i in 1:length(res$wgtStats)) {
            groupSum$"mean wgt"[groupSum$Level == i] <- res$wgtStats[[i]]$mean
            groupSum$"sum wgt"[groupSum$Level == i] <- res$wgtStats[[i]]$sum
        }
        res$groupSum <- groupSum
        
        M <- length(yvars)
        co0 <- (1/M) * Reduce("+", lapply(results, function(r) {
            coef(r)
        }))
        res$B <- (1/(M - 1)) * Reduce("+", lapply(results, function(r) {
            co <- coef(r) - co0
            outer(co, co)
        }))
        res$Ubar <- (1/M) * Reduce("+", lapply(results, function(r) {
            r$cov_mat
        }))
        res$VC <- res$Ubar + ((M + 1)/M) * res$B
        lnl <- (1/M) * Reduce("+", lapply(results, function(r) {
            r$lnl
        })) #calculate population SD like MPlus
        lnlsd <- sqrt((M-1)/M) *sd(unlist(lapply(results, function(r) {
            lo <- r$lnl
        })))
        AIC <- (1/M) * Reduce("+", lapply(results, function(r) {
            r$AIC
        })) #calculate population SD like MPlus
        AICsd <- sqrt((M-1)/M) *sd(unlist(lapply(results, function(r) {
            lo <- r$AIC
        })))
        BIC <- (1/M) * Reduce("+", lapply(results, function(r) {
            r$BIC
        })) #calculate population SD like MPlus
        BICsd <- sqrt((M-1)/M) *sd(unlist(lapply(results, function(r) {
            lo <- r$BIC
        })))
        FI_mean <- c(lnl,AIC,BIC)
        names(FI_mean) <- c("Loglikelihood","AIC","BIC")
        FI_sd <- c(lnlsd,AICsd,BICsd)
        names(FI_sd) <- c("Loglikelihood","AIC","BIC")
        FI <- cbind(FI_mean,FI_sd)
        colnames(FI) <- c("Mean","Std Dev")
        res$FI <- FI
        res$lnl <- NULL
        res$AIC <- NULL
        res$BIC <- NULL
        res$lnlf <- NULL
        res$CMODE <- NULL
        res$CMEAN <- NULL
        res$hessian <- NULL
        res$SE <- NULL
        res$call <- call
        res$PVresults <- results
        env <- environment(results[[1]]$lnlf)
        # Coefficients are just average value
        avg_coef <- rowSums(matrix(sapply(results, function(x) {
            x$coef
        }), nrow = length(results[[1]]$coef)))/length(yvars)
        names(avg_coef) <- names(results[[1]]$coef)
        
        # Calculates degrees of freedom
        var_between <- sum(res$varDF[which(!is.na(res$varDF$var1) & 
                                               is.na(res$varDF$var2)), "vcov"])
        var_total <- res$varDF$vcov[res$varDF$grp == "Residual"]
        M <- length(yvars)
        imputation_var <- ((M + 1)/((M - 1) * M)) * rowSums(matrix(sapply(results, 
                                                                          function(x) {
                                                                              x$coef - avg_coef
                                                                          })^2, nrow = length(avg_coef)))
        
        lambda <- (var_between + (var_between/M))/var_total
        res$dfold <- (M-1)/(lambda^2)
        res$coef <- avg_coef
        
        sampling_var <- colSums(Reduce(rbind, variances))/length(yvars)
        names(sampling_var) <- c(names(avg_coef), names(results[[1]]$vars))
        res$se <- sqrt(sampling_var[1:length(imputation_var)] + 
                           imputation_var)
        res$ICC <- tryCatch(sum(sapply(results, function(x) {
            x$ICC
        }))/length(yvars), error = function(cond) {
            return(NA)
        })
        varsmat0 <- results[[1]]$varDF
        m <- length(yvars)
        varsmat0$vcov <- rowSums(matrix(sapply(results, function(x) {
            x$varDF$vcov
        }), nrow = nrow(results[[1]]$varDF)))/m
        varsmat <- varsmat0[is.na(varsmat0$var2), c("level", 
                                                    "grp", "var1", "vcov", "SEvcov")]
        varsmat$st <- sqrt(varsmat$vcov)
        varsmat$t_value <- varsmat$vcov/varsmat$SEvcov[1:length(varsmat$vcov)]
        varsmat$p_value <- 2 * pt(-abs(varsmat$t_value),res$dfold)
        colnames(varsmat) <- c("Level", "Group", "Name", "Variance", 
                               "Std. Error", "Std.Dev.","t value","P-value")
        for (li in 2:max(varsmat0$level)) {
            varVC <- lapply(results, function(x) {
                vc <- as.matrix(x$varVC[[li]])
                cr <- atanh(cov2cor(vc))
                diag(cr) <- diag(vc)
                return(cr)
            })
            varVC <- Reduce("+", varVC)/length(varVC)
            cr <- tanh(varVC)
            if (ncol(cr) > 1) {
                for (i in 2:ncol(cr)) {
                    for (j in 1:(i - 1)) {
                        varsmat[varsmat$Level == li & varsmat$Name == 
                                    rownames(cr)[i], paste0("Corr", j)] <- cr[i, 
                                                                              j]
                    }
                }
            }
        }
        res$varsmatSum <- varsmat
        res$vars <- varsmat[, 4:6]
        rownames(res$vars) <- names(results[[1]]$vars)
        colnames(res$vars) <- colnames(results[[1]]$varDF)[4:6]
        res$varDF$vcov <- varsmat0$vcov
        imputation_var_for_vars <- ((M + 1)/((M - 1) * M)) * 
            apply(sapply(results, function(x) {
                x$varDF$vcov
            }), 1, function(x) {
                sum((x - mean(x))^2)
            })
        sampling_var_for_vars <- apply(sapply(results, function(x) {
            x$varDF$SEvcov^2
        }), 1, mean)
        res$varDF$SEvcov <- sqrt(sampling_var_for_vars + imputation_var_for_vars)
        res$se <- c(res$se, sqrt(sampling_var_for_vars + imputation_var_for_vars))
        res$Vimp <- c(imputation_var, imputation_var_for_vars)
        res$Vjrr <- sampling_var
        varn <- unlist(lapply(1:nrow(results[[1]]$varDF), function(ii) {
            paste(na.omit(unlist(results[[1]]$varDF[ii, 1:3])), 
                  collapse = ".")
        }))
        names(res$Vjrr) <- c(names(res$Vjrr)[1:(length(res$Vjrr) - 
                                                    length(varn))], varn)
        names(res$Vimp) <- names(res$Vjrr)
        names(res$se) <- names(res$Vjrr)
        res$formula <- mula
    }
    
    t_value <- res$coef/res$se[1:length(res$coef)]
    p_value <- 2 * pt(-abs(t_value),res$dfold)
    res$coef <- cbind(Estimate = res$coef, `Std. Error` = res$se[1:length(res$coef)], 
                      `t value` = t_value, `P-value` = p_value)
    res$npv <- length(yvars)
    res$n0 <- rawN
    res$nUsed <- nrow(edf)
    ngrp <- res$varDF
    ngrp <- ngrp[, c("grp", "ngrp", "level")]
    ngrp <- ngrp[!duplicated(ngrp$level), ]
    names(ngrp) <- c("Group Var", "Observations", "Level")
    res$ngroups <- ngrp
    nullOut <- c("ranefs", "theta", "invHessian", "is_adaptive", 
                 "sigma", "cov_mat", "varDF", "varVC", "var_theta", "PVresults", 
                 "SE")
    for (ni in 1:length(nullOut)) {
        res[[nullOut[ni]]] <- NULL
    }
    #class(res) <- "mixedSdfResults"
    class(res) <- "EFECTASMultilevel"
    return(res)
}
