# ---------------------------------------------------------------------------------------------------------------------------
# Regression
# ---------------------------------------------------------------------------------------------------------------------------

line_regression <-  function(mydata, myvariables, depended, independed, num_pack){
    formula <- as.formula(paste(depended, paste(independed, collapse=" + "), sep=" ~ "))
    if(num_pack == 1){
        #BIFIEsurvey
        if((any(grepl("pv",depended))== TRUE) | (any(grepl("pv",independed))== TRUE)){
            bdat <- BIFI_data(mydata)
        } else {
            bdat <- BIFI_data_pv(mydata, myvariables)
        }
        regression <- BIFIE.linreg(BIFIEobj = bdat, formula = formula, se = FALSE)
    } else if (num_pack == 2) {
        #EdSurvey
        regression <- lm.sdf(formula = formula, data = Pisa.data, weightVar = "w_fstuwt")
    } else {
        #intsvy
        if((any(grepl("pv",depended))== TRUE) | (any(grepl("pv",independed))== TRUE)){
            regresion <- pisa2015.reg(y = depended, x = independed, by = "cnt", data = mydata)
        } else {
            colnames(mydata) <- toupper(colnames(mydata))
            depended <- toupper(depended)
            independed <- toupper(independed)
            regression <- pisa2015.reg.pv(x = independed, pvlabel = depended, by = "CNT", data = mydata)
            colnames(mydata) <- tolower(colnames(mydata))
        }
    }
    return(regression)
}

log_regression <-  function(mydata, myvariables, depended, independed){
    formula <- as.formula(paste(depended, paste(independed, collapse=" + "), sep=" ~ "))
    #BIFIEsurvey
    if((any(grepl("pv",depended))== TRUE) | (any(grepl("pv",independed))== TRUE)){
        bdat <- BIFI_data(mydata)
    } else {
        bdat <- BIFI_data_pv(mydata, myvariables)
    }
    regression <- BIFIE.logistreg(BIFIEobj = bdat, formula = formula, se = FALSE)
    regression <- regression[["stat"]]
    return(regression)
}

multi_regression <-  function(mydata, depended, independed){
    formula <- as.formula(paste(paste(depended, collapse = "|"), paste(independed, collapse=" + "), sep=" ~ "))
    #EdSurvey
    regression <- mvrlm.sdf(formula = formula, data = Pisa.data, weightVar = "w_fstuwt")
    return(regression)
}
